/**************************
	MaxFeedReader
	By Neo101
	Version 1.1.0
**************************/



var FeedLib = {
	
	requests: [],

	
	abortRequests: function(){
		for(var i=0; i<FeedLib.requests.length; i++){
			FeedLib.requests[i].abort()
		}
		FeedLib.requests = []
	},
	
	
	loadFeed : function(url, callBackFunction, infoObj){
		var callBack = {'onFailure': FeedLib.onFailure, 
						'onSuccess': FeedLib.onResponseLoadFeed,
						'callBackFunction':callBackFunction,
						'infoObj':infoObj,
						'url':url}
						
		var object = new XmlDomObj()
		object.connect(url, callBack)
		
		FeedLib.requests.push(object)
	},

	
	onResponseLoadFeed : function(object, callBack){
		var xmlObject = object.documentElement
		if(xmlObject == null){
			callBack.callBackFunction(null, 'null error', callBack.infoObj)
		}
		var parsedFeed = FeedLib.parseFeed(xmlObject)
		if(parsedFeed){
			parsedFeed.url = callBack.url
		}
		callBack.callBackFunction(parsedFeed, null, callBack.infoObj)
	},
	
	
	parseFeed : function(xmlObject){
		var node = xmlObject.selectNodes('/rss')
		if(node.length > 0){
			return FeedLib.parseRSS(xmlObject)
		}
		node = xmlObject.selectNodes('/feed')
		if(node.length > 0){
			return FeedLib.parseATOM(xmlObject)
		}
		node = xmlObject.selectNodes('/rdf:RDF')
		if(node.length > 0){
			return FeedLib.parseRDF(xmlObject)
		}
		
		return null
	},
	
	stripData: function(feedObj){
		feedObj.homepageUrl = htmlStripLib.stripAttribute(feedObj.homepageUrl)
		feedObj.homepageDomain = htmlStripLib.find(/^(http:\/\/[\w.]+)\/?/i, feedObj.homepageUrl, 1)
		feedObj.title = htmlStripLib.stripText(feedObj.title)
		return feedObj
	},
	
	stripFeedItem: function(item, domain){
		var result = {}
		result.link = htmlStripLib.stripAttribute(item.link)
		result.title = htmlStripLib.stripText(item.title.replace(/<[^<>]*>/g, ''))
		if(result.title == ''){
			result.title = '[No Title]'
		}
		result.text = htmlStripLib.stripHtml(item.text, domain)
		if(SHOW_AUTHOR){
			result.author = htmlStripLib.stripText(item.author)
		}
		result.date = item.date
		return result
	},
	
	parseRSS : function(rootNode){
		var result = {}
		result.title = FeedLib.getSingleNode(rootNode, ['/rss/channel/title'])
		result.homepageUrl = FeedLib.getSingleNode(rootNode, ['/rss/channel/link'])
		var nodes = FeedLib.getMultipleNodes(rootNode, ['/rss/channel/item'])
		result.nodes = []
		var item
		for(var i=0; i<nodes.length; i++){
			item = {}
			item.link = FeedLib.getSingleNode(nodes[i], ['link'])
			item.text = FeedLib.getSingleNode(nodes[i], ['content:encoded', 'content', 'description'])
			item.title = FeedLib.getSingleNode(nodes[i], ['title'])
			if(ENABLE_TITLE_IGNORE && IGNORE_TITLE_REG_EXP.test(item.title)){
				continue
			}
			if(SHOW_AUTHOR){
				item.author = FeedLib.getSingleNode(nodes[i], ['author'])
			}
			var date = FeedLib.getSingleNode(nodes[i], ['pubDate']) // dates like Sun, 23 Dec 2007 18:41:28 +0100
			if(date){
				item.date = new Date(date.replace(/CET/g,''))
			} else {
				date = FeedLib.getSingleNode(nodes[i], ['dc:date']) // dates like 2007-12-21T18:48:44+01:00
				if(date){
					item.date = FeedLib.parseISO8601Date(date)
				}
			}
			result.nodes.push(item)
		}
		return FeedLib.stripData(result)
	},
	
	
	parseATOM : function(rootNode){
		var result = {}
		result.title = FeedLib.getSingleNode(rootNode, ['/feed/title'])
		result.homepageUrl = FeedLib.getSingleNode(rootNode, ['/feed/link[@type="text/html"][@rel="alternate"]/@href'])
		var nodes = FeedLib.getMultipleNodes(rootNode, ['/feed/entry'])
		result.nodes = []
		var item
		for(var i=0; i<nodes.length; i++){
			item = {}
			item.link = FeedLib.getSingleNode(nodes[i], ['link[@type="text/html"][@rel="alternate"]/@href'])
			item.text = FeedLib.getSingleNode(nodes[i], ['text', 'content', 'summary'])
			item.title = FeedLib.getSingleNode(nodes[i], ['title'])
			if(ENABLE_TITLE_IGNORE && IGNORE_TITLE_REG_EXP.test(item.title)){
				continue
			}
			if(SHOW_AUTHOR){
				item.author = FeedLib.getSingleNode(nodes[i], ['author/name'])
			}
			item.date = FeedLib.getSingleNode(nodes[i], ['published', 'modified', 'issued', 'created'])
			if(item.date){
				item.date = FeedLib.parseISO8601Date(item.date)
			}
			result.nodes.push(item)
		}
		return FeedLib.stripData(result)
	},
	
	parseRDF : function(rootNode){
		var result = {}
		result.title = FeedLib.getSingleNode(rootNode, ['/rdf:RDF/channel/title'])
		result.homepageUrl = FeedLib.getSingleNode(rootNode, ['/rdf:RDF/channel/link'])
		var nodes = FeedLib.getMultipleNodes(rootNode, ['/rdf:RDF/item'])
		result.nodes = []
		var item
		for(var i=0; i<nodes.length; i++){
			item = {}
			item.link = FeedLib.getSingleNode(nodes[i], ['link'])
			item.text = FeedLib.getSingleNode(nodes[i], ['description'])
			item.title = FeedLib.getSingleNode(nodes[i], ['title'])
			if(ENABLE_TITLE_IGNORE && IGNORE_TITLE_REG_EXP.test(item.title)){
				continue
			}
			if(SHOW_AUTHOR){
				item.author = FeedLib.getSingleNode(nodes[i], ['dc:creator'])
			}
			item.date = FeedLib.getSingleNode(nodes[i], ['dc:date'])
			if(item.date){
				item.date = FeedLib.parseISO8601Date(item.date)
			}
			result.nodes.push(item)
		}
		return FeedLib.stripData(result)
	},
	
	
	getSingleNode : function(node, xpathArray){
		var result
		for(var i=0; i<xpathArray.length; i++){
			result = node.selectSingleNode(xpathArray[i])
			if(result != null){
				return result.text
			}
		}
		return null
	},
	
	
	getMultipleNodes : function(node, xpathArray){
		var result
		for(var i=0; i<xpathArray.length; i++){
			result = node.selectNodes(xpathArray[i])
			if(result != null){
				return result
			}
		}
		return null
	},
	
	// I did not create this function, but I can't remember where I got it from....
	parseISO8601Date : function(strDate){
		var regexp = "([0-9]{4})(\-|)([0-9]{2})(\-|)([0-9]{2})( |T)([0-9]{2})(\:|)([0-9]{2})(\:|)([0-9]{2})";
		var d = strDate.match(new RegExp(regexp));

		if(!d) return null; // added line
		
		var offset = 0;
		var date = new Date();

		if (d[1]) { date.setFullYear(d[1]); }
		if (d[3]) { date.setMonth(d[3] - 1); }
		if (d[5]) { date.setDate(d[5]); }
		if (d[7]) { date.setHours(d[7]); }
		if (d[9]) { date.setMinutes(d[9]); }
		if (d[11]) { date.setSeconds(d[11]); }

		d = strDate.match(/(\+|\-)([0-9]{2})(|:)(|[0-9]{2})/);
		if(d){
			if(d[2]) offset+=Number(d[2])*60;
			if(d[4]) offset+=Number(d[4]);
			if(d[1]=="+") offset = 0-offset;
		}

		offset -= date.getTimezoneOffset();
		time = (Number(date) + (offset * 60 * 1000));
		date.setTime(Number(time));
		return date;
	},
	
	
	
	/****************** Main Functions **********************/
	
	
	onFailure : function(object, callBack){
		callBack.callBackFunction(null, [object.parseError.reason], callBack.infoObj)
	}

}





var htmlStripLib = {

	stripAttribute: function(attribute){
		return htmlStripLib.replacer(attribute, {'"':'&quot;', '\'':'&prime;', '<': '&lt;', '>': '&gt;'})
	},
	
	stripText: function(text){
		return htmlStripLib.replacer(text, {'<': '&lt;', '>': '&gt;'})
	},
	
	stripTextFull: function(text){
		return htmlStripLib.replacer(text, {'<': '&lt;', '>': '&gt;', '&': '&amp;'})
	},
	
	stripHtml: function(html, domain, showImages){
		if(html == null) return ''
        
		var tagRegExp = /<(\/?[a-z0-9:_\-!]+)([^<]*?)(\/?)(>|$)/gi
		var hrefRegExp = /\bhref\s*=\s*((?:"[^"]*"|'[^']*'|[^'">\s]+))/i
		var srcRegExp = /\bsrc\s*=\s*((?:"[^"]*"|'[^']*'|[^'">\s]+))/i
		var heightRegExp = /\bheight=["']?\d["'\b]/i
		var widthRegExp = /\bwidth=["']?\d["'\b]/i
		var alignRegExp = /\balign=["']?(left|right|center)["'\b]/i
		var lineBrakeRegExp = /^\/?(br|p|table|div|tr)$/i
		var keepTagRegExp = /^\/?(b|i|strong|blockquote|li|ul|ol|em|code|cite|pre)$/i
		var headingRegExp = /^\/?(h\d)$/i
		
		html = html.replace(/<(script|select|style)[\s\S]*?<\/\1>/gi, '')
		
		html = html.replace(tagRegExp, function(fullMatch, tagname, rest, endTag) {
			tagname = tagname.toLowerCase()
			if(lineBrakeRegExp.test(tagname)){
				return '<br />'
			}
			else if(keepTagRegExp.test(tagname)){
				return '<' + tagname + '>'
			}
			else if(headingRegExp.test(tagname)){
				if(tagname.substring(0,1) == '/'){
					return '</h4>'
				} else {
					return '<h4>'
				}
			}
			else if(tagname == '/a'){
				return '</a> '
			}
			else if(tagname == 'a'){
				var href = htmlStripLib.find(hrefRegExp, rest, 1)
				if(href){
					href = href.replace(/^['"]\/? ?|['"]$|^\//g, '')
					href = htmlStripLib.stripAttribute(href)
					if(href.substring(0, 11) == 'javascript:'){
						return ' '
					}
					if(href.substring(0,4) != 'http'){
						if(!domain){
							return ' '
						}
						href = domain + '/' + href
					}
					return '<a href="' + href + '" target="_blank">'
				} else {
				   return ' '
				}
			}
			else if(tagname == 'img'){
				if(showImages == false){
					return '[image]'
				}
				var src = htmlStripLib.find(srcRegExp, rest, 1)
				if(src){
					if(REMOVE_SMALL_IMAGES && heightRegExp.test(rest) && widthRegExp.test(rest)){
						return ''
					}
					src = src.replace(/^['"]\/? ?|['"]$|^\//g, '')
					src = htmlStripLib.stripAttribute(src)
					if(src.substring(0,4) != 'http'){
						if(!domain){
							return ' '
						}
						src = domain + '/' + src
					}
					var align = htmlStripLib.find(alignRegExp, rest, 1)
					if(align){
						var attributes = 'align="' + align + '"'
					} else {
						var attributes = ''
					}
					return '<img src="' + src + '" ' + attributes + ' /> '
				} else {
					return ' '
				}
			} else {
				return ' '
			}
			
		})
        
		html = html.replace(/(<br \/>\s*){2,}/g, '<br /><br />')
		html = html.replace(/^(<br \/>)+|(<br \/>)+$/g, '') // remove br tags at the beginning and end of an article
		html = html.replace(/(<li>|<\/?blockquote>|<\/h4>)\s*<br \/>/g, '$1')
        
		html += htmlStripLib.addClosingTags(html, 
			['a', 'h4', 'b', 'i', 'strong', 'blockquote', 'li', 'ul', 'ol', 'em', 'code', 'cite', 'pre'])
        
		return html
	},
	
	addClosingTags: function(text, tagsArray){
		var result = ''
		for(var i=0; i<tagsArray.length; i++){
			var startTagCount = htmlStripLib.count(text, '<' + tagsArray[i] + '\\b')
			if(startTagCount == 0){ // no need to check for the endtag anymore
				continue
			}
			var endTagCount = htmlStripLib.count(text, '</' + tagsArray[i] + '>')
			var difference = startTagCount - endTagCount
			for(var j=0; j<difference; j++){
				result += '</' + tagsArray[i] + '>'
			}
		}
		return result
	},
	
	find: function(regExp, text, group){
		var result = text.match(regExp)
		if(result != null){
			return result[group]
		}
		return null
	},
	
	replacer: function(text, regExpObj){
		if(!text) return text
		for(var regExpText in regExpObj){
			text = text.replace(new RegExp(regExpText, 'gi'), regExpObj[regExpText])
		}
		return text
	},
	
	count: function(text, regExpText){
		var match = text.match(new RegExp(regExpText, 'g'))
		if(match){
			return match.length
		}
		return 0
	}

}