/**************************
	MaxFeedReader
	By Neo101
	Version 1.1.0
**************************/

var keyboardLib = {
	
	timer: null,
	
	keysToCheck : {
		'J': goToNextItem,
		'S': goToNextFeed,
		'M': collapseExpandSidebar,
		'K': goToPreviousItem,
		'A': goToPreviousFeed,
		'F': goToNextFolder,
		'O': openLink
	},
	
	checkKeys : function(){
		if(keyboardLib.timer){
			return
		}
		for(var key in keyboardLib.keysToCheck){
			if(key == String.fromCharCode(window.event.keyCode)){
				keyboardLib.timer = window.setTimeout( function(){ 
					keyboardLib.timer = null;
					keyboardLib.keysToCheck[key]()
				}, 10)
				return
			}
		}
	}
}


var messageLib = {

	timeout: null,
	
	setMessage: function(text, displayTime){
		$setInnerHtml('message', text)
		try{
			window.clearTimeout(messageLib.timeout)
		} catch(e){}
		messageLib.timeout = window.setTimeout(messageLib.clearMessage, (displayTime || 5) * 1000)
	},

	clearMessage: function(){
		try{
			window.clearTimeout(messageLib.timeout)
		} catch(e){}
		$setInnerHtml('message', '')
	}
}


var focusLib = {
	lastSelectedElementId: null,
	focusedItem: null,
	
	getFocusId: function(){
		var split = focusLib.lastSelectedElementId.split('_')
		if(split && split.length == 2){
			return parseInt(split[1])
		}
		return null
	}

}



document.onkeydown = keyboardLib.checkKeys

var OFFSET = 30 // in pixels, used for top header



function goTo(element, offset){
	var offset = offset || 0
	offset -= OFFSET
	var positionY = element.getBoundingClientRect().top
	window.scrollBy(0, positionY + offset)
}


function goToNextItem(){
	var id, element
	var loopAllItems = true
	
	if(focusLib.lastSelectedElementId && scrolledToBottom()){
		var currentNumber = focusLib.getFocusId()
		if(currentNumber != null){
			id = 'item_' + (currentNumber + 1)
			element = $id(id)
			if(!element){ // at end of list
				loopAllItems = false
			}
			else if(element.getBoundingClientRect().top < 0 + OFFSET){ // element is above the currently visible items
				loopAllItems = true
				element = null
			} else { // element found
				loopAllItems = false
			}
		}
	}

	if(loopAllItems){
		var start = 0
		if(focusLib.lastSelectedElementId){
			var position = $id(focusLib.lastSelectedElementId).getBoundingClientRect().top - OFFSET
			if(position < 20){ // no need to find the item from the start
				start = focusLib.getFocusId() || 0
			}
		}
		for(var i=start; i<MaxFeedReader.itemId; i++){
			id = 'item_' + i
			var iElement = $id(id)
			if(!iElement){ // tried all items in list
				break
			}
			if(iElement.getBoundingClientRect().top > 5 + OFFSET){ // first visible item found
				element = iElement
				break
			}
		}
	}

	if(element){ //lastSelectedElementId != element.id
		focusItem(element)
	} else {
		messageLib.setMessage('No next item found', 3)
	}
}


function scrolledToBottom(){
	return (document.documentElement.scrollHeight == document.documentElement.scrollTop + document.documentElement.clientHeight || document.documentElement.scrollHeight <  document.documentElement.clientHeight)
}


function goToPreviousItem(){
	var id, element
	var loopAllItems = true
	
	if(focusLib.lastSelectedElementId && scrolledToBottom()){
		var currentNumber = focusLib.getFocusId()
		if(currentNumber != null){
			id = 'item_' + (currentNumber - 1)
			element = document.getElementById(id)
			if(!element){ // at end of list
				loopAllItems = false
			}
			else if(element.getBoundingClientRect().top < 0 + OFFSET){ // element is above the currently visible items
				loopAllItems = true
				element = null
			} else { // element found
				loopAllItems = false
			}
		}
	}

	
	if(loopAllItems){
		var start = MaxFeedReader.itemId-1
		if(focusLib.lastSelectedElementId){
			var position = $id(focusLib.lastSelectedElementId).getBoundingClientRect().top - OFFSET
			if(position > 0){ // no need to find the item from the start
				start = focusLib.getFocusId() || MaxFeedReader.itemId-1
			}
		}
		for(var i=start; i>=0; i--){
			element = $id('item_' + i)
			if(element.getBoundingClientRect().top < 0 + OFFSET){ // first visible item found
				break
			}
			if(i == 0){ // already at begin of list
				element = null
			}
		}
	}
	
	if(element){
		focusItem(element)
	} else {
		messageLib.setMessage('No previous item found', 3)
	}
}


function focusItem(element){
	focusLib.lastSelectedElementId = element.id
	try{
		focusLib.focusedItem.style.textDecoration = 'none'
	} catch(e){}
	try{
		focusLib.focusedItem = element.childNodes[0]
		focusLib.focusedItem.focus()
		focusLib.focusedItem.style.textDecoration = 'underline'
	} catch(e){}
	goTo(element, -5)
}


function openLink(){
	try{
		focusLib.focusedItem.click()
	} catch(e){}
}


function goToNextFeed(){
	var nodes = $tag('h2')
	var id
	for(var i=0; i<nodes.length; i++){
		if(nodes[i].getBoundingClientRect().top > OFFSET){
			id = nodes[i].id
			break
		}
	}
	if(id){
		goTo($id(id))
	} else {
		messageLib.setMessage('No next feed found', 3)
	}
}


function goToPreviousFeed(){
	var nodes = $tag('h2')
	var id
	for(var i=nodes.length-1; i >= 0; i--){
		if(nodes[i].getBoundingClientRect().top < 0){
			id = nodes[i].id
			break
		}
	}
	if(id){
		goTo($id(id))
	} else {
		messageLib.setMessage('No previous feed found', 3)
	}
}


function goToNextFolder(){
	var node = $id('feedList')
	var nodes = node.getElementsByTagName('span')
	var getNext = false
	for(var i=0; i < nodes.length; i++){
		if(nodes[i].className == 'folder'){
			if(MaxFeedReader.loadedFolder == '' || getNext){
				MaxFeedReader.loadFolder(nodes[i].innerText, false)
				return
			} else if(nodes[i].innerText == MaxFeedReader.loadedFolder){
				if(i < nodes.length){
					getNext = true
				} else {
					messageLib.setMessage('nonext')
				}
			}
		}
	}
}


function collapseExpandSidebar(){
	if($id('sidebar').style.display == 'none'){
		$show('sidebar')
		$id('main').style.paddingLeft = '200px'
	} else {
		$hide('sidebar')
		$id('main').style.paddingLeft = '0px'
	}
}


function expandCollapse(headElement, noWrite, self){
	var element = headElement.nextSibling
	var elementId = element.id
	var itemName
	if(elementId){
		itemName = 'Expand' + elementId
	}

	if(element.style.display == 'none'){
		$show(element)
		self.innerHTML = '<img src="images/minus.gif" alt="Collapse" />'
		if(noWrite == null){
			saveState(itemName, '1')
		}
	}
	else{
		$hide(element)
		self.innerHTML = '<img src="images/plus.gif" alt="Expand" />'
		if(noWrite == null){
			saveState(itemName, '0')
		}
	}
}

function showHideHelp(){
	if($id('help').style.display == 'none'){
		$show('help')
	} else {
		$hide('help')
	}
}
